<?php

namespace Drupal\field_tools\Form;

use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Serialization\Yaml;
use Drupal\field_tools\FieldCloner;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a form to export multiple fields to config YAML.
 */
class ConfigFieldsExportToYamlForm extends FormBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity type bundle info service.
   *
   * @var \Drupal\Core\Entity\EntityTypeBundleInfoInterface
   */
  protected $entityTypeBundleInfo;

  /**
   * The field cloner.
   *
   * @var \Drupal\field_tools\FieldCloner
   */
  protected $fieldCloner;

  /**
   * Creates a Clone instance.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Entity\EntityTypeBundleInfoInterface $entity_type_bundle_info
   *   The entity type bundle info service.
   * @param \Drupal\field_tools\FieldCloner $field_cloner
   *   The field cloner.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, EntityTypeBundleInfoInterface $entity_type_bundle_info, FieldCloner $field_cloner) {
    $this->entityTypeManager = $entity_type_manager;
    $this->entityTypeBundleInfo = $entity_type_bundle_info;
    $this->fieldCloner = $field_cloner;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('entity_type.bundle.info'),
      $container->get('field_tools.field_cloner')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'field_tools_field_export_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $entity_type_id = NULL, $bundle = NULL) {
    $this->entityTypeId = $entity_type_id;
    $this->bundle = $bundle;

    $field_manager = \Drupal::service('entity_field.manager');
    // TODO: only config fields!
    $field_definitions = $field_manager->getFieldDefinitions($entity_type_id, $bundle);

    $code = [];
    if (!empty($form_state->getValues())) {
      $code = [];

      foreach (array_filter($form_state->getValues()['fields']) as $field_name) {
        $this->getConfigYaml($code, $field_definitions[$field_name]);
        // TODO: bundle fields and config field conversion.

        $code[] = '';
      }

      $form['code'] = [
        '#type' => 'textarea',
        '#value' => implode("\n", $code),
        '#rows' => count($code) + 1,
      ];
    }

    $field_options = [];
    foreach ($field_definitions as $field_id => $field) {
      $field_options[$field_id] = $this->t("@field-label (machine name: @field-name)", array(
        '@field-label' => $field->getLabel(),
        '@field-name' => $field->getName(),
      ));
    }
    asort($field_options);

    $form['fields'] = array(
      '#title' => $this->t('Fields to export to YAML'),
      '#type' => 'checkboxes',
      '#options' => $field_options,
      '#description' => $this->t("Select fields to export as config YAML."),
    );

    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => $this->t('Export field definitions to YAML'),
    );

    return $form;
  }

  protected function getConfigYaml(&$code, FieldDefinitionInterface $field) {
    $this->configStorage = \Drupal::service('config.storage');

    $field_storage = $field->getFieldStorageDefinition();

    $code[] = '_config_prefix: ' . $field_storage->getEntityType()->getConfigPrefix();
    $code[] = Yaml::encode($this->configStorage->read($field_storage->getConfigDependencyName()));

    $code[] = '_config_prefix: ' . $field->getEntityType()->getConfigPrefix();
    $code[] = Yaml::encode($this->configStorage->read($field->getConfigDependencyName()));
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $form_state->setRebuild();
  }

}
